/*
* RTCP packet dumper.
* Usage:
*   getrtcp -v address/port
* Listens on port pair specified by argument for RTCP
* packets.
*  -v: show hex dump
*
* ( This program is based on the RTP/RTCP dump program rtpdump,
* which is basically implemented by H. Schulzrinne, GMD Fokus
* rtpdump - Version 1.0 )
*
* This program will only dump RTCP control information
*
*/

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/time.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include "types.h"
#include "rtp.h"

typedef u_int32 member_t;

static int verbose = 0; /* decode */

static void hex(char *buf, int len)
{
  int i;

  for (i = 0; i < len; i++) {
    printf("%02x ", (unsigned char)buf[i]);
  }
  printf("\n");
} /* hex */



void member_sdes(member_t m, rtcp_sdes_type_t t, char *b, int len)
{
  static struct {
    rtcp_sdes_type_t t;
    char *name;
  } map[] = {
    {RTCP_SDES_END,   "end"}, 
    {RTCP_SDES_CNAME, "CNAME"},
    {RTCP_SDES_NAME,  "NAME"},
    {RTCP_SDES_EMAIL, "EMAIL"},
    {RTCP_SDES_PHONE, "PHONE"},
    {RTCP_SDES_LOC,   "LOC"},
    {RTCP_SDES_TOOL,  "TOOL"},
    {RTCP_SDES_TXT,   "TXT"},
    {RTCP_SDES_PRIV,  "PRIV"},
    {0,0}
  };
  int i;

  for (i = 0; map[i].name; i++) {
    if (map[i].t == t) break;
  }
  printf("#%0x %-5.5s(%d): %*.*s\n", m, 
    map[i].name ? map[i].name : "????", t, len, len, b);
}




static void rtcp_parse(char *buf, int len)
{
  rtcp_t *r;         /* RTCP header */
  char *s;
  int i;

  r = (rtcp_t *)buf;
  if (r->common.type != RTP_TYPE) {
    printf("\n#Invalid type %d\n", r->common.type);
    return;
  }

  while (len > 0) {
    len -= (r->common.length + 1) << 2;
    if (len < 0) {
      /* something wrong with packet format */
      printf("\n#Illegal length %d\n", r->common.length);
      break;
    }
    switch (r->common.pt) {
    case RTCP_SR:
      if ( r->common.count == 1 ) { // Only one host is an active sender
        printf("%u.%u BYTES=%u LOSS=%f\n",
		ntohl(r->r.sr.ntp_sec),
		ntohl(r->r.sr.ntp_frac) , 
		ntohl(r->r.sr.osent), 
                0); 
         fflush(stdout);
         return;
      }
      for (i = 0; i < r->common.count; i++) { 
        // Determine the  reception report refering to the local host
        if ( r->r.sr.rr[i].ssrc != r->r.rr.ssrc ) {
         printf("%u.%u BYTES=%u LOSS=%f\n",
		ntohl(r->r.sr.ntp_sec),
		ntohl(r->r.sr.ntp_frac) , 
		ntohl(r->r.sr.osent), 
                ((float)(ntohl(r->r.sr.rr[i].fraction)) /256.0*100.0) ); 
         fflush(stdout);
         return;
        }
       }
     break;

    default:
      /* invalid type */
    break;
    }
    r = (rtcp_t *)((u_int32 *)r + r->common.length + 1);
  }
} /* rtcp_parse */


main(int argc, char *argv[])
{
  int c;
  char *host;
  char *s;
  struct sockaddr_in sin;
  int sock;
  fd_set readfds;
  extern char *optarg;
  extern int optind;
  int port = 3456;
  int i;
  struct ip_mreq mreq;      /* multicast group */


  while ((c = getopt(argc, argv, "v")) != EOF) {
    switch(c) {
    case 'v':
      verbose = 1;
      break;
    case '?':
      fprintf(stderr, "Usage: %s [-v] [multicast]/port\n", argv[0]);
      exit(1);
      break;
    }
  }

  if (optind == argc) {
    fprintf(stderr, "Usage: %s [mc]/port\n", argv[0]);
    exit(1);
  }

  host = argv[optind];
  s = strchr(host, '/');
  if (*s) {
    *s = '\0';
    port = htons(atoi(s+1));
  }

  if (verbose) {
    printf("%s %s\n", argv[0], VERSION);
  }

  sin.sin_family = AF_INET;
  /* multicast */
  if (host) {
    sin.sin_addr.s_addr = inet_addr(host);
  }
  else {
    sin.sin_addr.s_addr = INADDR_ANY;
  }

  /* create/bind sockets */

    sock = socket(PF_INET, SOCK_DGRAM, 0);
    if (sock < 0) {
      perror("socket");
      exit(1);
    }

    sin.sin_port = htons(port + 1);
    if (bind(sock, &sin, sizeof(sin)) < 0) {
      perror("socket");
      exit(1);
    }

    if (IN_CLASSD(sin.sin_addr.s_addr)) {
      mreq.imr_multiaddr.s_addr = sin.sin_addr.s_addr;
      mreq.imr_interface.s_addr = htonl(INADDR_ANY);
      if (setsockopt(sock, IPPROTO_IP,
        IP_ADD_MEMBERSHIP, (char *)&mreq, sizeof(mreq)) < 0) {
          perror("IP_ADD_MEMBERSHIP");
          exit(1);
      }
    }

  /* main loop */
  while (1) {
    int len;
    char buf[8192];
    FD_ZERO(&readfds);
    FD_SET(sock, &readfds);
    c = select(sizeof(readfds), &readfds, 0, 0, 0);
    if (c < 0) {
      perror("select");
      exit(1);
    }    
      if (FD_ISSET(sock, &readfds)) {
        len = read(sock, buf, sizeof(buf));
        rtcp_parse(buf, len);
      }
  }
} /* main */
